@@ -32,6 +32,7 @@
 #endif // HARBOUR
 
 #ifdef MENUS
+#include "MenuHandler.hh"
 #include "PDecor.hh"
 #include "PMenu.hh"
 #include "WORefMenu.hh"
@@ -351,8 +352,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                 actionGotoClientID(it->getParamI(0));
                 break;
             case ACTION_EXEC:
-                if (it->getParamS().size())
-                    Util::forkExec(it->getParamS());
+                actionExec(client, it->getParamS());
                 break;
 #ifdef MENUS
             case ACTION_SHOW_MENU:
@@ -360,7 +360,7 @@ ActionHandler::handleAction(const ActionPerformed &ap)
                                ap.type, client ? client : ap.wo);
                 break;
             case ACTION_HIDE_ALL_MENUS:
-                WindowManager::instance()->hideAllMenus();
+                MenuHandler::instance()->hideAllMenus();
                 break;
 #endif // MENUS
             case ACTION_RELOAD:
@@ -518,8 +518,8 @@ ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, std
         return 0;
     }
 
-    state &= ~PScreen::instance()->getNumLock() & ~PScreen::instance()->getScrollLock() & ~LockMask;
-    state &= ~Button1Mask & ~Button2Mask & ~Button3Mask & ~Button4Mask & ~Button5Mask;
+    PScreen::stripStateModifiers(&state);
+    PScreen::stripButtonModifiers(&state);
 
     list<ActionEvent>::iterator it(actions->begin());
     for (; it != actions->end(); ++it) {
@@ -533,6 +533,18 @@ ActionHandler::findMouseAction(uint button, uint state, MouseEventType type, std
     return 0;
 }
 
+/**
+ * Execute action, setting client environment before (if any).
+ */
+void
+ActionHandler::actionExec(Client *client, const std::string &command)
+{
+    if (command.size()) {
+        Client::setClientEnvironment(client);
+        Util::forkExec(command);
+    }
+}
+
 //! @brief Searches for a client matching titles and makes it visible
 void
 ActionHandler::actionFindClient(const std::wstring &title)
@@ -659,27 +671,17 @@ ActionHandler::actionFocusToggle(uint button, uint raise, int off,
 
     // find the focused window object
     PWinObj *fo_wo = 0;
-    if (PWinObj::getFocusedPWinObj()) {
-        if (PWinObj::getFocusedPWinObj()->getType() == PWinObj::WO_CLIENT) {
-            fo_wo = PWinObj::getFocusedPWinObj()->getParent();
+    if (PWinObj::isFocusedPWinObj(PWinObj::WO_CLIENT)) {
+        fo_wo = PWinObj::getFocusedPWinObj()->getParent();
 
-            list<PMenu::Item*>::iterator it(menu->m_begin());
-            for (; it != menu->m_end(); ++it) {
-                if ((*it)->getWORef() == fo_wo) {
-                    menu->selectItem(it);
-                    break;
-                }
+        list<PMenu::Item*>::iterator it(menu->m_begin());
+        for (; it != menu->m_end(); ++it) {
+            if ((*it)->getWORef() == fo_wo) {
+                menu->selectItem(it);
+                break;
             }
-            fo_wo->setFocused(false);
-
-        } else {
-#ifdef DEBUG
-            cerr << __FILE__ << "@" << __LINE__ << ": "
-                 << "ActionHandler(" << this << ")::actionFocusToggle("
-                 << button << "," << raise << "," << off << "," << mru << ")"
-                 << endl << " *** focused PWinObj != WO_FRAME" << endl;
-#endif // DEBUG
         }
+        fo_wo->setFocused(false);
     }
 
     if (Config::instance()->getShowFrameList()) {
@@ -836,7 +838,7 @@ void
 ActionHandler::actionShowMenu(const std::string &name, bool stick,
                               uint e_type, PWinObj *wo_ref)
 {
-    PMenu *menu = WindowManager::instance()->getMenu(name);
+    PMenu *menu = MenuHandler::instance()->getMenu(name);
     if (! menu) {
         return;
     }
