@@ -26,6 +26,7 @@ extern "C" {
 #endif // HAVE_SHAPE
 }
 
+#include "Compat.hh" // setenv, unsetenv
 #include "PWinObj.hh"
 #include "PDecor.hh" // PDecor::TitleItem
 #include "Client.hh"
@@ -65,64 +66,29 @@ Client::Client(Window new_client, bool is_new)
       _alive(false), _marked(false),
       _send_focus_message(false), _send_close_message(false),
       _wm_hints_input(true), _cfg_request_lock(false),
-#ifdef HAVE_SHAPE
-      _shaped(false),
-#endif // HAVE_SHAPE
-      _extended_net_name(false)
+      _shaped(false), _extended_net_name(false)
 {
-    // Construct the client
-    PScreen::instance()->grabServer();
-
-    // PWinObj attributes
+    // PWinObj attributes, required by validate etc.
     _window = new_client;
     _type = WO_CLIENT;
 
+    // Construct the client
+    PScreen::instance()->grabServer();
+    if (! validate() || ! getAndUpdateWindowAttributes()) {
+        PScreen::instance()->ungrabServer(true);
+        return;
+    }
+
     // Get unique Client id
     _id = findClientID();
     _title.setId(_id);
     _title.infoAdd(PDecor::TitleItem::INFO_ID);
 
-    if (! validate()) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": " << "Couldn't validate client." << endl;
-#endif // DEBUG
-        PScreen::instance()->ungrabServer(true);
-        return;
-    }
-
-    XWindowAttributes attr;
-    if (! XGetWindowAttributes(_dpy, _window, &attr)) {
-#ifdef DEBUG
-        cerr << __FILE__ << "@" << __LINE__ << ": " << "Client died." << endl;
-#endif // DEBUG
-        PScreen::instance()->ungrabServer(true);
-        return;
-    }
-    _gm.x = attr.x;
-    _gm.y = attr.y;
-    _gm.width = attr.width;
-    _gm.height = attr.height;
-
-    _cmap = attr.colormap;
-    _size = XAllocSizeHints();
-
-    // get trans window and the window attributes
-    getTransientForHint();
-
-    XSetWindowAttributes sattr;
-    sattr.event_mask =
-        PropertyChangeMask|StructureNotifyMask|FocusChangeMask;
-    sattr.do_not_propagate_mask =
-        ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
-
-    // We don't want these masks to be propagated down to the frame
-    XChangeWindowAttributes(_dpy, _window, CWEventMask|CWDontPropagate, &sattr);
-
-#ifdef HAVE_SHAPE
     if (PScreen::instance()->hasExtensionShape()) {
+#ifdef HAVE_SHAPE
         XShapeSelectInput(_dpy, _window, ShapeNotifyMask);
-    }
 #endif // HAVE_SHAPE
+    }
 
     XAddToSaveSet(_dpy, _window);
     XSetWindowBorderWidth(_dpy, _window, 0);
@@ -139,145 +105,29 @@ Client::Client(Window new_client, bool is_new)
     // cyclic dependency, getting the name requires quiering autoprops
     _class_hint->title = _title.getReal();
 
-    // Get Autoproperties before EWMH as we need the cfg_deny property.
+    // Get Autoproperties before EWMH as we need the cfg_deny
+    // property, however the _transient hint needs to be setup to
+    // avoid auto-grouping to be to greedy.
+    getTransientForHint();
     AutoProperty *ap = readAutoprops(WindowManager::instance()->isStartup()
                                      ? APPLY_ON_NEW : APPLY_ON_START);
 
-    readMwmHints(); // read atoms
-    readEwmhHints();
-    readPekwmHints();
-    readIcon();
-    readClientPid();
-    readClientRemote();
-    getWMProtocols();
+    readHints();
+    // We need to set the state before acquiring a frame,
+    // so that Frame's state can match the state of the Client.
+    setInitialState();
+    findOrCreateFrame(ap);
 
-    ulong initial_state = NormalState;
-    XWMHints* hints = XGetWMHints(_dpy, _window);
-    if (hints) {
-        // get the input focus mode
-        if (hints->flags&InputHint) { // FIXME: More logic needed
-            _wm_hints_input = hints->input;
-        }
-
-        // Get initial state of the window
-        if (hints->flags&StateHint) {
-            initial_state = hints->initial_state;
-        }
-
-        XFree(hints);
-    }
-
-    // Set state either specified in hint
-    if (getWmState() == IconicState) {
-        _iconified = true;
-    }
-
-    if (_iconified || initial_state == IconicState) {
-        _iconified = true;
-        _mapped = true;
-        unmapWindow();
-    } else {
-        setWmState(initial_state);
-    }
-
-    // grab keybindings and mousebutton actions
+    // Grab keybindings and mousebutton actions
     KeyGrabber::instance()->grabKeys(_window);
     grabButtons();
 
-    // don't want to spread the giveInputFocuses
-    bool do_focus = is_new ? Config::instance()->isFocusNew() : false;
-    bool do_autogroup = true;
-
-    // If pekwm is already running, check against autoproperty then
-    // tagged frame. If starting up, check if it has a FRAME_ID and if not
-    // use autoproperties.
-    if (WindowManager::instance()->isStartup()) {
-        // Check for tagged frame
-        Frame *frame = Frame::getTagFrame();
-        if (frame && frame->isMapped()) {
-            _parent = frame;
-            frame->addChild(this);
-
-            if (! Frame::getTagBehind()) {
-                frame->activateChild(this);
-                do_focus = frame->isFocused();
-            }
-        }
-
-        // Not startup, pekwm (re)start it is.
-    } else {
-        long id;
-        if (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ID), id)) {
-            do_autogroup = false;
-            _parent = Frame::findFrameFromID(id);
-            if (_parent) {
-                Frame *frame = static_cast<Frame*>(_parent);
-                frame->addChild(this);
-                frame->activateChild(this);
-                do_focus = frame->isFocused();
-            }
-        }
-    }
-
-
-    // Apply Autoproperties again to override EWMH state. It's done twice as
-    // we need the cfg_deny property when reading the EWMH state.
-    if (ap) {
-        applyAutoprops(ap);
-
-        if (do_autogroup && ! _parent && (ap->group_size >= 0)) {
-            Frame* frame = WindowManager::instance()->findGroup(ap);
-            if (frame) {
-                frame->addChild(this);
-
-                if (! ap->group_behind) {
-                    frame->activateChild(this);
-                    do_focus = frame->isFocused();
-                }
-                if (ap->group_raise) {
-                    frame->raise();
-                }
-            }
-        }
-    }
-
     // Tell the world about our state
     updateEwmhStates();
 
     PScreen::instance()->ungrabServer(true);
 
-    // if we don't have a frame already, create a new one
-    if (! _parent) {
-        _parent = new Frame(this, ap);
-    }
-
-    // Make sure the window is mapped, this is done after it has been
-    // added to the decor/frame as otherwise IsViewable state won't
-    // be correct and we don't know whether or not to place the window
-    if (! _iconified) {
-        PWinObj::mapWindow();
-    }
-
-    // Let us hear what autoproperties has to say about focusing
-    if (is_new && ap && ap->isMask(AP_FOCUS_NEW)) {
-        do_focus = ap->focus_new;
-    }
-
-    // Only can give input focus to mapped windows
-    if (_parent->isMapped()) {
-        // Ordinary focus
-        if (do_focus) {
-            _parent->giveInputFocus();
-
-            // Check if we are transient, and if we want to focus
-        } else if ((_transient != None)
-                   && Config::instance()->isFocusNewChild()) {
-            Client *trans_for = findClientFromWindow(_transient);
-            if (trans_for && trans_for->isFocused()) {
-                _parent->giveInputFocus();
-            }
-        }
-    }
+    setMappedStateAndFocus(is_new, ap);
 
     _alive = true;
 
@@ -299,22 +149,6 @@ Client::~Client(void)
 
     PScreen::instance()->grabServer();
 
-#ifdef MENUS
-    WORefMenu *wo_ref_menu;
-
-    wo_ref_menu = static_cast<WORefMenu*>(WindowManager::instance()->getMenu("WINDOW"));
-    if (this == wo_ref_menu->getWORef()) {
-        wo_ref_menu->setWORef(0);
-        wo_ref_menu->unmapAll();
-    }
-
-    wo_ref_menu = static_cast<WORefMenu*>(WindowManager::instance()->getMenu("DECORMENU"));
-    if (this == wo_ref_menu->getWORef()) {
-        wo_ref_menu->setWORef(0);
-        wo_ref_menu->unmapAll();
-    }
-#endif // MENUS
-
     // removes gravity and moves it back to root if we are alive
     bool focus = false;
     if (_parent && (_parent->getType() == PWinObj::WO_FRAME)) {
@@ -359,6 +193,198 @@ Client::~Client(void)
     PScreen::instance()->ungrabServer(true);
 }
 
+/**
+ * Read basic window attributes including geometry and update the
+ * window attributes being listened to. Returns false if the client
+ * disappears during the check.
+ */
+bool
+Client::getAndUpdateWindowAttributes(void)
+{
+    XWindowAttributes attr;
+    if (! XGetWindowAttributes(_dpy, _window, &attr)) {
+        return false;
+    }
+    _gm.x = attr.x;
+    _gm.y = attr.y;
+    _gm.width = attr.width;
+    _gm.height = attr.height;
+
+    _cmap = attr.colormap;
+    _size = XAllocSizeHints();
+
+    XSetWindowAttributes sattr;
+    sattr.event_mask =
+        PropertyChangeMask|StructureNotifyMask|FocusChangeMask;
+    sattr.do_not_propagate_mask =
+        ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
+
+    // We don't want these masks to be propagated down to the frame
+    XChangeWindowAttributes(_dpy, _window, CWEventMask|CWDontPropagate, &sattr);
+
+    return true;
+}
+
+/**
+ * Find frame for client based on tagging, hints and
+ * autoproperties. Create a new one if not found and add the client.
+ */
+void
+Client::findOrCreateFrame(AutoProperty *autoproperty)
+{
+    if (! _parent) {
+        findTaggedFrame();
+    }
+    if (! _parent) {
+        findPreviousFrame();
+    }
+
+    // Apply Autoproperties again to override EWMH state. It's done twice as
+    // we need the cfg_deny property when reading the EWMH state.
+    if (autoproperty != 0) {
+        applyAutoprops(autoproperty);
+        if (! _parent) {
+            findAutoGroupFrame(autoproperty);
+        }
+    }
+
+    // if we don't have a frame already, create a new one
+    if (! _parent) {
+        _parent = new Frame(this, autoproperty);
+    }
+}
+
+/**
+ * Find from client from for the currently tagged client.
+ */
+bool
+Client::findTaggedFrame(void)
+{
+    if (! WindowManager::instance()->isStartup()) {
+        return false;
+    }
+
+    // Check for tagged frame
+    Frame *frame = Frame::getTagFrame();
+    if (frame && frame->isMapped()) {
+        _parent = frame;
+        frame->addChild(this);
+
+        if (! Frame::getTagBehind()) {
+            frame->activateChild(this);
+        }
+    }
+
+    return _parent != 0;
+}
+
+/**
+ * Find frame for client on PEKWM_FRAME_ID hint.
+ */
+bool
+Client::findPreviousFrame(void)
+{
+    if (WindowManager::instance()->isStartup()) {
+        return false;
+    }
+
+    long id;
+    if (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ID), id)) {
+        _parent = Frame::findFrameFromID(id);
+        if (_parent) {
+            Frame *frame = static_cast<Frame*>(_parent);
+            frame->addChildOrdered(this);
+            if (getPekwmFrameActive()) {
+                frame->activateChild(this);
+            }
+        }
+    }
+
+    return _parent != 0;
+}
+
+/**
+ * Find frame for client based on autoproperties.
+ */
+bool
+Client::findAutoGroupFrame(AutoProperty *autoproperty)
+{
+    if (autoproperty->group_size < 0) {
+        return false;
+    }
+
+    Frame* frame = WindowManager::instance()->findGroup(autoproperty);
+    if (frame) {
+        frame->addChild(this);
+
+        if (! autoproperty->group_behind) {
+            frame->activateChild(this);
+        }
+        if (autoproperty->group_raise) {
+            frame->raise();
+        }
+    }
+
+    return _parent != 0;
+}
+
+/**
+ * Get the client state and set iconified and mapped flags.
+ */
+void
+Client::setInitialState(void)
+{
+    // Set state either specified in hint
+    ulong initial_state = readWmHints();
+    if (getWmState() == IconicState) {
+        _iconified = true;
+    }
+
+    if (_iconified || initial_state == IconicState) {
+        _iconified = true;
+        _mapped = true;
+        unmapWindow();
+    } else {
+        setWmState(initial_state);
+    }
+}
+
+/**
+ * Ensure the Client is (un) mapped and give input focus if requested.
+ */
+void
+Client::setMappedStateAndFocus(bool is_new, AutoProperty *autoproperty)
+{
+    // Make sure the window is mapped, this is done after it has been
+    // added to the decor/frame as otherwise IsViewable state won't
+    // be correct and we don't know whether or not to place the window
+    if (! _iconified) {
+        PWinObj::mapWindow();
+    }
+
+    // Let us hear what autoproperties has to say about focusing
+    bool do_focus = is_new ? Config::instance()->isFocusNew() : false;
+    if (is_new && autoproperty && autoproperty->isMask(AP_FOCUS_NEW)) {
+        do_focus = autoproperty->focus_new;
+    }
+
+    // Only can give input focus to mapped windows
+    if (_parent->isMapped()) {
+        // Ordinary focus
+        if (do_focus) {
+            _parent->giveInputFocus();
+
+            // Check if we are transient, and if we want to focus
+        } else if ((_transient != None)
+                   && Config::instance()->isFocusNewChild()) {
+            Client *trans_for = findClientFromWindow(_transient);
+            if (trans_for && trans_for->isFocused()) {
+                _parent->giveInputFocus();
+            }
+        }
+    }
+}
+
 // START - PWinObj interface.
 
 //! @brief Maps the window.
@@ -544,6 +570,16 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
     return 0;
 }
 
+//!@brief
+ActionEvent*
+Client::handleMapRequest(XMapRequestEvent *ev)
+{
+       if (_parent && dynamic_cast<PDecor *>(_parent)) {
+           dynamic_cast<PDecor*>(_parent)->deiconify();
+       }
+       return 0;
+}
+
 // END - PWinObj interface.
 
 //! @brief Finds the Client which holds the Window w.
@@ -689,6 +725,45 @@ Client::setStateCfgDeny(StateAction sa, uint deny)
     }
 }
 
+/**
+ * Read "all" hints required during creation of Client.
+ */
+void
+Client::readHints(void)
+{
+    readMwmHints(); // read atoms
+    readEwmhHints();
+    readPekwmHints();
+    readIcon();
+    readClientPid();
+    readClientRemote();
+    getWMProtocols();
+}
+
+/**
+ * Read WM Hints, return the initial state of the window.
+ */
+ulong
+Client::readWmHints(void)
+{
+    ulong initial_state = NormalState;
+    XWMHints* hints = XGetWMHints(_dpy, _window);
+    if (hints) {
+        // get the input focus mode
+        if (hints->flags&InputHint) { // FIXME: More logic needed
+            _wm_hints_input = hints->input;
+        }
+
+        // Get initial state of the window
+        if (hints->flags&StateHint) {
+            initial_state = hints->initial_state;
+        }
+
+        XFree(hints);
+    }
+    return initial_state;
+}
+
 /**
  * Read the WM_CLASS hint and set information on the _class_hint used
  * in matching auto properties.
@@ -842,6 +917,8 @@ Client::readPekwmHints(void)
     if (AtomUtil::getString(_window, Atoms::getAtom(PEKWM_TITLE), str)) {
         _title.setUser(Util::to_wide_str(str));
     }
+
+    _state.initial_frame_order = getPekwmFrameOrder();
 }
 
 //! @brief Read _NET_WM_ICON from client window.
@@ -1609,12 +1686,15 @@ Client::getWMProtocols(void)
     }
 }
 
-//! @brief
+/**
+ * Read WM_TRANSIENT_FOR hint.
+ */
 void
 Client::getTransientForHint(void)
 {
-    if (! _transient)
+    if (_transient == None) {
         XGetTransientForHint(_dpy, _window, &_transient);
+    }
 }
 
 //! @brief
@@ -1655,3 +1735,59 @@ Client::removeStrutHint(void)
     delete _strut;
     _strut = 0;
 }
+
+/**
+ * Get _PEKWM_FRAME_ORDER hint from client, return < 0 on failure.
+ */
+long
+Client::getPekwmFrameOrder(void)
+{
+    long num = -1;
+    AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ORDER), num);
+    return num;
+}
+
+/**
+ * Update _PEKWM_FRAME_ORDER hint on client window.
+ */
+void
+Client::setPekwmFrameOrder(long num)
+{
+    AtomUtil::setLong(_window, Atoms::getAtom(PEKWM_FRAME_ORDER), num);
+}
+
+/**
+ * Get _PEKWM_FRAME_ACTIVE hint from client window, return true if
+ * client is treated as active.
+ */
+bool
+Client::getPekwmFrameActive(void)
+{
+    long act = 0;
+    return (AtomUtil::getLong(_window, Atoms::getAtom(PEKWM_FRAME_ACTIVE), act)
+            && act == 1);
+}
+
+/**
+ * Set _PEKWM_FRAME_ACTIVE hint on client window.
+ */
+void
+Client::setPekwmFrameActive(bool act)
+{
+    AtomUtil::setLong(_window, Atoms::getAtom(PEKWM_FRAME_ACTIVE), act ? 1 : 0);
+}
+
+/**
+ * Update the environment with CLIENT_PID and CLIENT_WINDOW.
+ */
+void
+Client::setClientEnvironment(Client *client)
+{
+    if (client) {
+        setenv("CLIENT_PID", Util::to_string<long>(client->isRemote() ? -1 : client->getPid()).c_str(), 1);
+        setenv("CLIENT_WINDOW", Util::to_string<Window>(client->getWindow()).c_str(), 1);
+    } else {
+        unsetenv("CLIENT_PID");
+        unsetenv("CLIENT_WINDOW");
+    }
+}
