@@ -12,6 +12,7 @@
 
 #include <algorithm>
 #include <cstdio>
+#include <cwchar>
 #include <functional>
 #include <iostream>
 
@@ -49,6 +50,7 @@ using std::mem_fun;
 using std::string;
 using std::vector;
 using std::wstring;
+using std::swprintf;
 
 list<Frame*> Frame::_frame_list = list<Frame*>();
 vector<uint> Frame::_frameid_list = vector<uint>();
@@ -59,7 +61,7 @@ bool Frame::_tag_behind = false;
 //! @brief Frame constructor
 Frame::Frame(Client *client, AutoProperty *ap)
     : PDecor(WindowManager::instance()->getScreen()->getDpy(), WindowManager::instance()->getTheme(),
-             Frame::getClientDecorName(client)),
+             Frame::getClientDecorName(client), client->getWindow()),
       _id(0), _client(0), _class_hint(0),
       _non_fullscreen_decor_state(0), _non_fullscreen_layer(LAYER_NORMAL)
 {
@@ -391,29 +393,48 @@ Frame::handleUnmapEvent(XUnmapEvent *ev)
 
 // END - PWinObj interface.
 
-#ifdef HAVE_SHAPE
-//! @brief
 void
 Frame::handleShapeEvent(XAnyEvent *ev)
 {
+#ifdef HAVE_SHAPE
     if (! _client || (ev->window != _client->getWindow())) {
         return;
     }
     _client->setShaped(setShape());
-}
 #endif // HAVE_SHAPE
+}
+
 
 // START - PDecor interface.
 
 //! @brief Adds child to the frame.
 void
-Frame::addChild (PWinObj *child)
+Frame::addChild (PWinObj *child, std::list<PWinObj*>::iterator *it)
 {
-    PDecor::addChild(child);
+    PDecor::addChild(child, it);
     AtomUtil::setLong(child->getWindow(), Atoms::getAtom(PEKWM_FRAME_ID), _id);
     child->lower();
 }
 
+/**
+ * Add child preserving order from the previous pekwm run.
+ */
+void
+Frame::addChildOrdered (Client *child)
+{
+    Client *client;
+    
+    list<PWinObj*>::iterator it(_child_list.begin());
+    for (; it != _child_list.end(); ++it) {
+        client = static_cast<Client*>(*it);
+        if (child->getInitialFrameOrder() < client->getInitialFrameOrder()) {
+            break;
+        }
+    }
+
+    addChild(child, &it);
+}
+
 //! @brief Removes child from the frame.
 void
 Frame::removeChild (PWinObj *child, bool do_delete)
@@ -441,11 +462,9 @@ Frame::activateChild (PWinObj *child)
 
     // setShape uses current active child, so we need to activate the
     // child before setting shape
-#ifdef HAVE_SHAPE
     if (PScreen::instance()->hasExtensionShape()) {
         _client->setShaped(setShape());
     }
-#endif // HAVE_SHAPE
 
     if (_focused) {
         child->giveInputFocus();
@@ -457,21 +476,29 @@ Frame::activateChild (PWinObj *child)
     Workspaces::instance()->updateClientStackingList(true, true);
 }
 
-//! @brief
+/**
+ * Called when child order is updated, re-sets the titles and updates
+ * the _PEKWM_FRAME hints.
+ */
 void
 Frame::updatedChildOrder(void)
 {
     titleClear();
 
+    Client *client;
     list<PWinObj*>::iterator it(_child_list.begin());
-    for (; it != _child_list.end(); ++it) {
-        titleAdd(static_cast<Client*>(*it)->getTitle());
+    for (long num = 0; it != _child_list.end(); ++num, ++it) {
+        client = static_cast<Client*>(*it);
+        client->setPekwmFrameOrder(num);
+        titleAdd(client->getTitle());
     }
 
     updatedActiveChild();
 }
 
-//! @brief
+/**
+ * Set the active title and update the _PEKWM_FRAME hints.
+ */
 void
 Frame::updatedActiveChild(void)
 {
@@ -479,9 +506,9 @@ Frame::updatedActiveChild(void)
 
     list<PWinObj*>::iterator it(_child_list.begin());
     for (uint i = 0; it != _child_list.end(); ++i, ++it) {
+        static_cast<Client*>(*it)->setPekwmFrameActive(_child == *it);
         if (_child == *it) {
             titleSetActive(i);
-            break;
         }
     }
 
@@ -1093,6 +1120,7 @@ Frame::doResize(bool left, bool x, bool top, bool y)
     }
 
     setShaded(STATE_UNSET); // make sure the frame isn't shaded
+    setStateFullscreen(STATE_UNSET);
 
     // Initialize variables
     int start_x, new_x;
@@ -1348,6 +1376,7 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
     }
 
     setShaded(STATE_UNSET);
+    setStateFullscreen(STATE_UNSET);
 
     // make sure the two states are in sync if toggling
     if ((horz == vert) && (sa == STATE_TOGGLE)) {
@@ -1436,7 +1465,7 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
     }
 
     fixGeometry(); // harbour already considered
-    downSize(true, true); // keep x and keep y ( make conform to inc size )
+    downSize(_gm, true, true); // keep x and keep y ( make conform to inc size )
 
     moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
@@ -1725,7 +1754,7 @@ Frame::growDirection(uint direction)
         break;
     }
 
-    downSize((direction != DIRECTION_LEFT), (direction != DIRECTION_UP));
+    downSize(_gm, (direction != DIRECTION_LEFT), (direction != DIRECTION_UP));
 
     moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 }
@@ -1900,9 +1929,11 @@ Frame::calcGravityPosition(int gravity, int x, int y, int &g_x, int &g_y)
     }
 }
 
-//! @brief Makes Frame conform to Clients width and height inc
+/**
+ * Makes gm conform to _clients width and height inc.
+ */
 void
-Frame::downSize(bool keep_x, bool keep_y)
+Frame::downSize(Geometry &gm, bool keep_x, bool keep_y)
 {
     XSizeHints *size_hint = _client->getXSizeHints(); // convenience
 
@@ -1913,9 +1944,9 @@ Frame::downSize(bool keep_x, bool keep_y)
                   ? size_hint->base_width
                   : (size_hint->flags&PMinSize) ? size_hint->min_width : 0;
 
-        _gm.width -= (getChildWidth() - b_x) % size_hint->width_inc;
+        gm.width -= (getChildWidth() - b_x) % size_hint->width_inc;
         if (! keep_x) {
-            _gm.x = o_r - _gm.width;
+            gm.x = o_r - gm.width;
         }
     }
 
@@ -1926,9 +1957,9 @@ Frame::downSize(bool keep_x, bool keep_y)
                   ? size_hint->base_height
                   : (size_hint->flags&PMinSize) ? size_hint->min_height : 0;
 
-        _gm.height -= (getChildHeight() - b_y) % size_hint->height_inc;
+        gm.height -= (getChildHeight() - b_y) % size_hint->height_inc;
         if (! keep_y) {
-            _gm.y = o_b - _gm.height;
+            gm.y = o_b - gm.height;
         }
     }
 }
@@ -1990,125 +2021,72 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
 void
 Frame::handleConfigureRequestGeometry(XConfigureRequestEvent *ev, Client *client)
 {
-    // Look for fullscreen requests
-    const long all_geometry = CWX|CWY|CWWidth|CWHeight;
-    bool is_fullscreen = false;
-    if (Config::instance()->isFullscreenDetect()
-        && ! client->isCfgDeny(CFG_DENY_SIZE)
-        && ! client->isCfgDeny(CFG_DENY_POSITION)
-        && ((ev->value_mask&all_geometry) == static_cast<unsigned>(all_geometry))) {
-
-        Geometry gm_request(ev->x, ev->y, ev->width, ev->height);
-        if (gm_request == _scr->getScreenGeometry()
-             || gm_request == _scr->getHeadGeometry(_scr->getNearestHead(ev->x, ev->y))) {
-
-            is_fullscreen = true;
-            setStateFullscreen(STATE_SET);
-        }
+    if (isRequestGeometryFullscreen(ev, client)) {
+        client->configureRequestSend();
+        return;
     }
 
-    if (! is_fullscreen) {
-        bool change_geometry = false;
+    bool change_geometry = false;
+    if (! client->isCfgDeny(CFG_DENY_SIZE)
+        && (ev->value_mask & (CWWidth|CWHeight))) {
+        resizeChild(ev->width, ev->height);
+        _client->setShaped(setShape());
+        change_geometry = true;
+    }
 
-        if (! client->isCfgDeny(CFG_DENY_SIZE)
-                && (ev->value_mask & (CWWidth|CWHeight)) ) {
+    if (! client->isCfgDeny(CFG_DENY_POSITION)
+        && (ev->value_mask & (CWX|CWY)) ) {
+        calcGravityPosition(_client->getXSizeHints()->win_gravity,
+                            ev->x, ev->y, _gm.x, _gm.y);
+        move(_gm.x, _gm.y);
+        change_geometry = true;
+    }
 
-            resizeChild(ev->width, ev->height);
-            _client->setShaped(setShape());
-
-           change_geometry = true;
-        }
-
-        if (! client->isCfgDeny(CFG_DENY_POSITION)
-               && (ev->value_mask & (CWX|CWY)) ) {
-
-            calcGravityPosition(_client->getXSizeHints()->win_gravity,
-                                ev->x, ev->y, _gm.x, _gm.y);
-            move(_gm.x, _gm.y);
-
-           change_geometry = true;
-        }
-
-        // Remove fullscreen state if client changes it size
-        if (change_geometry && Config::instance()->isFullscreenDetect()) {
-            setStateFullscreen(STATE_UNSET);
-        }
+    // Remove fullscreen state if client changes it size
+    if (change_geometry && Config::instance()->isFullscreenDetect()) {
+        setStateFullscreen(STATE_UNSET);
     }
 }
 
-//! @brief
+/**
+ * Check if requested size if "fullscreen"
+ */
+bool
+Frame::isRequestGeometryFullscreen(XConfigureRequestEvent *ev, Client *client)
+{
+    bool is_fullscreen = false;
+    if (! client->isCfgDeny(CFG_DENY_SIZE)
+        && ! client->isCfgDeny(CFG_DENY_POSITION)) {
+        int nearest_head = _scr->getNearestHead(ev->x, ev->y);
+        Geometry gm_request(ev->x, ev->y, ev->width, ev->height);
+        Geometry gm_screen(_scr->getScreenGeometry());
+        Geometry gm_head(_scr->getHeadGeometry(nearest_head));
+
+        if (gm_request == gm_screen || gm_request == gm_head) {
+            is_fullscreen = true;
+        } else {
+            downSize(gm_screen, true, true);
+            downSize(gm_head, true, true);
+            if (gm_request == gm_screen || gm_request == gm_head) {
+                is_fullscreen = true;
+            }
+        }
+
+        if (is_fullscreen && Config::instance()->isFullscreenDetect()) {
+            setStateFullscreen(STATE_SET);
+        }
+    }
+    return is_fullscreen;
+}
+
+/**
+ * Handle client message.
+ */
 void
 Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
 {
-    StateAction sa;
-
     if (ev->message_type == Atoms::getAtom(STATE)) {
-        if (ev->data.l[0]== NET_WM_STATE_REMOVE) {
-            sa = STATE_UNSET;
-        } else if (ev->data.l[0]== NET_WM_STATE_ADD) {
-            sa = STATE_SET;
-        } else if (ev->data.l[0]== NET_WM_STATE_TOGGLE) {
-            sa = STATE_TOGGLE;
-        } else {
-#ifdef DEBUG
-            cerr << __FILE__ << "@" << __LINE__ << ": "
-                 << "None of StateAdd, StateRemove or StateToggle." << endl;
-#endif // DEBUG
-            return;
-        }
-
-#define IS_STATE(S) ((ev->data.l[1] == long(Atoms::getAtom(S))) || (ev->data.l[2] == long(Atoms::getAtom(S))))
-
-        // actions that only is going to be applied on the active client
-        if (client == _client) {
-            // there is no modal support in pekwm yet
-// 			if (IS_STATE(STATE_MODAL)) {
-// 				is_modal=true;
-// 			}
-            if (IS_STATE(STATE_STICKY)) {
-                setStateSticky(sa);
-            }
-            if (IS_STATE(STATE_MAXIMIZED_HORZ)
-                    && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
-                setStateMaximized(sa, true, false, false);
-            }
-            if (IS_STATE(STATE_MAXIMIZED_VERT)
-                    && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
-                setStateMaximized(sa, false, true, false);
-            }
-            if (IS_STATE(STATE_SHADED)) {
-                setShaded(sa);
-            }
-            if (IS_STATE(STATE_HIDDEN)
-                    && ! client->isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
-                setStateIconified(sa);
-            }
-            if (IS_STATE(STATE_FULLSCREEN)
-                    && ! client->isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
-                setStateFullscreen(sa);
-            }
-            if (IS_STATE(STATE_ABOVE)
-                    && ! client->isCfgDeny(CFG_DENY_STATE_ABOVE)) {
-                setStateAlwaysOnTop(sa);
-            }
-            if (IS_STATE(STATE_BELOW)
-                    && ! client->isCfgDeny(CFG_DENY_STATE_BELOW)) {
-                setStateAlwaysBelow(sa);
-            }
-        }
-
-        if (IS_STATE(STATE_SKIP_TASKBAR)) {
-            client->setStateSkip(sa, SKIP_TASKBAR);
-        }
-        if (IS_STATE(STATE_SKIP_PAGER)) {
-            client->setStateSkip(sa, SKIP_PAGER);
-        }
-        if (IS_STATE(STATE_DEMANDS_ATTENTION)) {
-            client->setStateDemandsAttention(sa, true);
-        }
-
-        client->updateEwmhStates();
-
+        handleClientStateMessage(ev, client);
     } else if (ev->message_type == Atoms::getAtom(NET_ACTIVE_WINDOW)) {
         if (! client->isCfgDeny(CFG_DENY_ACTIVE_WINDOW)) {
             // Active child if it's not the active child
@@ -2119,7 +2097,8 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
             // If we aren't mapped we check if we make sure we're on the right
             // workspace and then map the window.
             if (! _mapped) {
-                if (_workspace != Workspaces::instance()->getActive()) {
+                if (_workspace != Workspaces::instance()->getActive() &&
+                        !isSticky()) {
                     Workspaces::instance()->setWorkspace(_workspace, false);
                 }
                 mapWindow();
@@ -2140,9 +2119,99 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
             iconify();
         }
     }
-#undef IS_STATE
 }
 
+/**
+ * Handle _NET_WM_STATE atom.
+ */
+void
+Frame::handleClientStateMessage(XClientMessageEvent *ev, Client *client)
+{
+    StateAction sa = getStateActionFromMessage(ev);
+    handleStateAtom(sa, ev->data.l[1], client);
+    if (ev->data.l[2] != 0) {
+        handleStateAtom(sa, ev->data.l[2], client);
+    }
+    client->updateEwmhStates();
+}
+
+/**
+ * Get StateAction from NET_WM atom.
+ */
+StateAction
+Frame::getStateActionFromMessage(XClientMessageEvent *ev)
+{
+    StateAction sa = STATE_SET;
+    if (ev->data.l[0]== NET_WM_STATE_REMOVE) {
+        sa = STATE_UNSET;
+    } else if (ev->data.l[0]== NET_WM_STATE_ADD) {
+        sa = STATE_SET;
+    } else if (ev->data.l[0]== NET_WM_STATE_TOGGLE) {
+        sa = STATE_TOGGLE;
+    }
+    return sa;
+}
+
+/**
+ * Handle state atom for client.
+ */
+void
+Frame::handleStateAtom(StateAction sa, Atom atom, Client *client)
+{
+    if (client == _client) {
+        handleCurrentClientStateAtom(sa, atom, client);
+    }
+
+    switch (atom) {
+    case STATE_SKIP_TASKBAR:
+        client->setStateSkip(sa, SKIP_TASKBAR);
+        break;
+    case STATE_SKIP_PAGER:
+        client->setStateSkip(sa, SKIP_PAGER);
+        break;
+    case STATE_DEMANDS_ATTENTION:
+        client->setStateDemandsAttention(sa, true);
+        break;
+    }
+}
+
+/**
+ * Handle state atom for actions that apply only on active client.
+ */
+void
+Frame::handleCurrentClientStateAtom(StateAction sa, Atom atom, Client *client)
+{
+    if (atom == Atoms::getAtom(STATE_STICKY)) {
+        setStateSticky(sa);
+    }
+    if (atom == Atoms::getAtom(STATE_MAXIMIZED_HORZ)
+            && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_HORZ)) {
+        setStateMaximized(sa, true, false, false);
+    }
+    if (atom == Atoms::getAtom(STATE_MAXIMIZED_VERT)
+            && ! client->isCfgDeny(CFG_DENY_STATE_MAXIMIZED_VERT)) {
+        setStateMaximized(sa, false, true, false);
+    }
+    if (atom == Atoms::getAtom(STATE_SHADED)) {
+        setShaded(sa);
+    }
+    if (atom == Atoms::getAtom(STATE_HIDDEN)
+            && ! client->isCfgDeny(CFG_DENY_STATE_HIDDEN)) {
+        setStateIconified(sa);
+    }
+    if (atom == Atoms::getAtom(STATE_FULLSCREEN)
+            && ! client->isCfgDeny(CFG_DENY_STATE_FULLSCREEN)) {
+        setStateFullscreen(sa);
+    }
+    if (atom == Atoms::getAtom(STATE_ABOVE)
+            && ! client->isCfgDeny(CFG_DENY_STATE_ABOVE)) {
+        setStateAlwaysOnTop(sa);
+    }
+    if (atom == Atoms::getAtom(STATE_BELOW)
+            && ! client->isCfgDeny(CFG_DENY_STATE_BELOW)) {
+        setStateAlwaysBelow(sa);
+    }
+}
 
 //! @brief
 void
