@@ -11,6 +11,8 @@
 #endif // HAVE_CONFIG_H
 
 #include <list>
+#include <cwchar>
+#include <cwctype>
 
 #include "InputDialog.hh"
 #include "KeyGrabber.hh"
@@ -25,15 +27,15 @@ extern "C" {
 
 using std::list;
 using std::wstring;
+using std::iswprint;
 
 /**
  * InputDialog constructor.
  */
 InputDialog::InputDialog(Display *dpy, Theme *theme, const std::wstring &title)
-  : PDecor(dpy, theme, "INPUTDIALOG"),
-    _data(theme->getCmdDialogData()),
-    _pixmap_bg(None), _pos(0), _buf_off(0), _buf_chars(0),
-    _wo_ref(0)
+    : PDecor(dpy, theme, "INPUTDIALOG"), PWinObjReference(0),
+      _data(theme->getCmdDialogData()),
+      _pixmap_bg(None), _pos(0), _buf_off(0), _buf_chars(0)
 {
     // PWinObj attributes
     _layer = LAYER_NONE; // hack, goes over LAYER_MENU
