@@ -1,6 +1,6 @@
 //
 // PDecor.cc for pekwm
-// Copyright © 2004-2009 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2004-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -49,6 +49,7 @@ using std::map;
 using std::mem_fun;
 using std::string;
 using std::vector;
+using std::swprintf;
 
 // PDecor::Button
 
@@ -128,7 +129,7 @@ PDecor::Button::setState(ButtonState state)
             XShapeCombineRectangles(_dpy, _window, ShapeBounding,
                                     0, 0, &rect, 1, ShapeSet, YXBanded);
         }
-#endif
+#endif // HAVE_SHAPE
         clear();
     }
 }
@@ -183,7 +184,8 @@ list<PDecor*> PDecor::_pdecor_list = list<PDecor*>();
 //! @param dpy Display
 //! @param theme Theme
 //! @param decor_name String, if not DEFAULT_DECOR_NAME sets _decor_name_override
-PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
+PDecor::PDecor(Display *dpy, Theme *theme,
+               const std::string decor_name, const Window child_window)
     : PWinObj(dpy),
       _theme(theme),_decor_name(decor_name),
       _child(0), _button(0), _button_press_win(None),
@@ -212,35 +214,16 @@ PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
         _data = _theme->getPDecorData(DEFAULT_DECOR_NAME);
     }
 
-    XSetWindowAttributes attr;
-    attr.override_redirect = True;
-    attr.event_mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|
-                      EnterWindowMask|SubstructureRedirectMask|SubstructureNotifyMask;
-    _window =
-        XCreateWindow(_dpy, PScreen::instance()->getRoot(),
-                      _gm.x, _gm.y, _gm.width, _gm.height, 0,
-                      CopyFromParent, InputOutput, CopyFromParent,
-                      CWOverrideRedirect|CWEventMask, &attr);
-
-    attr.event_mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask;
-    attr.override_redirect = True;
-    _title_wo.setWindow(XCreateWindow(_dpy, _window,
-                                      borderLeft(), borderTop(), 1, 1, 0, // don't know our size yet
-                                      CopyFromParent, InputOutput, CopyFromParent,
-                                      CWOverrideRedirect|CWEventMask, &attr));
-    addChildWindow(_title_wo.getWindow());
-
-    // create border windows
-    for (uint i = 0; i < BORDER_NO_POS; ++i) {
-        attr.cursor = ScreenResources::instance()->getCursor(ScreenResources::CursorType(i));
-
-        _border_win[i] =
-            XCreateWindow(_dpy, _window, -1, -1, 1, 1, 0,
-                          CopyFromParent, InputOutput, CopyFromParent,
-                          CWOverrideRedirect|CWEventMask|CWCursor, &attr);
-        _border_pos_map[BorderPosition(i)] = None;
-        addChildWindow(_border_win[i]);
+    CreateWindowParams window_params;
+    getParentWindowAttributes(window_params, child_window);
+    createParentWindow(window_params);
+    if (window_params.mask & CWColormap) {
+        window_params.depth = PScreen::instance()->getDepth();
+        window_params.visual = PScreen::instance()->getVisual()->getXVisual();
+        window_params.attr.colormap = PScreen::instance()->getColormap();
     }
+    createTitle(window_params);
+    createBorder(window_params);
 
     // sets buttons etc up
     loadDecor();
@@ -251,6 +234,97 @@ PDecor::PDecor(Display *dpy, Theme *theme, const std::string decor_name)
     _pdecor_list.push_back(this);
 }
 
+/**
+ * Create window attributes
+ */
+void
+PDecor::getParentWindowAttributes(CreateWindowParams &params,
+                                  Window child_window)
+{
+    params.mask = CWOverrideRedirect|CWEventMask|CWBorderPixel|CWBackPixel;
+    params.depth = CopyFromParent;
+    params.visual = CopyFromParent;
+    params.attr.override_redirect = True;
+    params.attr.border_pixel = 0;
+    params.attr.background_pixel = 0;
+
+    if (child_window != None) {
+        getChildWindowAttributes(params, child_window);
+    }
+}
+
+/**
+ * Get window attributes from child window
+ */
+void
+PDecor::getChildWindowAttributes(CreateWindowParams &params,
+                                 Window child_window)
+{
+    XWindowAttributes attr;
+    Status status =  XGetWindowAttributes(_dpy, child_window, &attr);
+    if (status != BadDrawable && status != BadWindow && attr.depth == 32) {
+        params.mask |= CWColormap;
+        params.depth = attr.depth;
+        params.visual = attr.visual;
+        params.attr.colormap = XCreateColormap(_dpy,
+                                               PScreen::instance()->getRoot(),
+                                               params.visual, AllocNone);
+    }
+}
+
+/**
+ * Create container window.
+ */
+void
+PDecor::createParentWindow(CreateWindowParams &params)
+{
+    params.attr.event_mask = ButtonPressMask|ButtonReleaseMask|
+        ButtonMotionMask|EnterWindowMask|SubstructureRedirectMask|
+        SubstructureNotifyMask;
+    _window = XCreateWindow(_dpy, PScreen::instance()->getRoot(),
+                            _gm.x, _gm.y, _gm.width, _gm.height, 0,
+                            params.depth, InputOutput, params.visual,
+                            params.mask, &params.attr);
+}
+
+/**
+ * Create title window.
+ */
+void
+PDecor::createTitle(CreateWindowParams &params)
+{
+    params.attr.event_mask = ButtonPressMask|ButtonReleaseMask|
+        ButtonMotionMask|EnterWindowMask;
+    Window title = XCreateWindow(_dpy, _window,
+                                 borderLeft(), borderTop(), 1, 1, 0,
+                                 params.depth, InputOutput, params.visual,
+                                 params.mask, &params.attr);
+    _title_wo.setWindow(title);
+    addChildWindow(_title_wo.getWindow());
+}
+
+/**
+ * Create border windows
+ */
+void
+PDecor::createBorder(CreateWindowParams &params)
+{
+    params.attr.event_mask = ButtonPressMask|ButtonReleaseMask|
+        ButtonMotionMask|EnterWindowMask;
+
+    ScreenResources *sr = ScreenResources::instance();
+    for (uint i = 0; i < BORDER_NO_POS; ++i) {
+        params.attr.cursor = sr->getCursor(ScreenResources::CursorType(i));
+
+        _border_win[i] =
+            XCreateWindow(_dpy, _window, -1, -1, 1, 1, 0,
+                          params.depth, InputOutput, params.visual,
+                          params.mask|CWCursor, &params.attr);
+        _border_pos_map[BorderPosition(i)] = None;
+        addChildWindow(_border_win[i]);
+    }
+}
+
 //! @brief PDecor destructor
 PDecor::~PDecor(void)
 {
@@ -534,8 +608,8 @@ PDecor::handleButtonPress(XButtonEvent *ev)
     list<ActionEvent> *actions = 0;
 
     // Remove state modifiers from event
-    ev->state = PScreen::instance()->stripStateModifiers(ev->state);
-    ev->state = PScreen::instance()->stripButtonModifiers(ev->state);
+    PScreen::stripStateModifiers(&ev->state);
+    PScreen::stripButtonModifiers(&ev->state);
 
     // Try to do something about frame buttons
     if (ev->subwindow != None && (_button = findButton(ev->subwindow)) != 0) {
@@ -618,8 +692,8 @@ PDecor::handleButtonRelease(XButtonEvent *ev)
     MouseEventType mb = MOUSE_EVENT_RELEASE;
 
     // Remove state modifiers from event
-    ev->state = PScreen::instance()->stripStateModifiers(ev->state);
-    ev->state = PScreen::instance()->stripButtonModifiers(ev->state);
+    PScreen::stripStateModifiers(&ev->state);
+    PScreen::stripButtonModifiers(&ev->state);
 
     // handle titlebar buttons
     if (_button) {
@@ -993,10 +1067,14 @@ PDecor::getTitleHeight(void) const
 
 //! @brief Adds a child to the decor, reparenting the window
 void
-PDecor::addChild(PWinObj *child)
+PDecor::addChild(PWinObj *child, std::list<PWinObj*>::iterator *it)
 {
     child->reparent(this, borderLeft(), borderTop() + getTitleHeight());
-    _child_list.push_back(child);
+    if (it == 0) {
+        _child_list.push_back(child);
+    } else {
+        _child_list.insert(*it, child);
+    }
 
     updatedChildOrder();
 
@@ -1469,6 +1547,16 @@ PDecor::setSkip(uint skip)
     _skip = skip;
 }
 
+//! @brief Remove iconified state.
+void PDecor::deiconify(void) {
+    if (_iconified) {
+        if (_workspace == Workspaces::instance()->getActive()) {
+            mapWindow();
+        }
+        _iconified = false;
+    }
+}
+
 //! @brief Renders and sets title background
 void
 PDecor::renderTitle(void)
