@@ -42,8 +42,9 @@ using std::wstring;
 using std::find;
 
 PMenu::Item::Item(const std::wstring &name, PWinObj *wo_ref, PTexture *icon)
-  : _x(0), _y(0), _name(name), _wo_ref(wo_ref),
-    _type(MENU_ITEM_NORMAL), _icon(icon), _creator(0)
+    : PWinObjReference(wo_ref),
+      _x(0), _y(0), _name(name), 
+      _type(MENU_ITEM_NORMAL), _icon(icon), _creator(0)
 {
     if (_icon) {
         TextureHandler::instance()->referenceTexture(_icon);
@@ -247,7 +248,7 @@ PMenu::handleMotionEvent(XMotionEvent *ev)
         handleItemEvent(button ? MOUSE_EVENT_MOTION_PRESSED : MOUSE_EVENT_MOTION, ev->x, ev->y);
 
         ActionEvent *ae;
-        ev->state = PScreen::instance()->stripButtonModifiers(ev->state);
+        PScreen::stripButtonModifiers(&ev->state);
         ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION,
                                             Config::instance()->getMouseActionList(MOUSE_ACTION_LIST_MENU));
 
@@ -423,6 +424,11 @@ PMenu::buildMenuCalculate(void)
     _separator_height = _theme->getMenuData()->getTextureSeparator(OBJECT_STATE_FOCUSED)->getHeight();
 
     height = (_item_height * _size) + (_separator_height * sep);
+
+    if (_size) {
+    	_size += sep;
+    }
+
     buildMenuCalculateColumns(width, height);
 
     // Check if we need to enable scrolling
@@ -517,12 +523,17 @@ PMenu::buildMenuCalculateColumns(unsigned int &width, unsigned int &height)
         list<PMenu::Item*>::iterator it(_item_list.begin());
         for (i = 0, it = _item_list.begin(); i < _cols; ++i) {
             row_height = 0;
-            for (j = 0; (j < _rows) && (it != _item_list.end()); ++it) {
-                if ((*it)->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
+            for (j = 0; (j < _rows) && (it != _item_list.end()); ++it, ++j) {
+                switch ((*it)->getType()) {
+                case PMenu::Item::MENU_ITEM_NORMAL:
                     row_height += _item_height;
-                    ++j; // only count real menu items
-                } else if ((*it)->getType() == PMenu::Item::MENU_ITEM_SEPARATOR) {
+                    break;
+                case PMenu::Item::MENU_ITEM_SEPARATOR:
                     row_height += _separator_height;
+                    break;
+                case PMenu::Item::MENU_ITEM_HIDDEN:
+                default:
+                	break;
                 }
             }
 
