@@ -1,6 +1,6 @@
 //
 // PScreen.cc for pekwm
-// Copyright © 2003-2009 Claes Nästén <me{@}pekdon{.}net>
+// Copyright © 2003-2009 Claes Nästén <me@pekdon.net>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -13,6 +13,7 @@
 #include <string>
 #include <iostream>
 #include <cassert>
+#include <cstring> // required for memset in FD_ZERO
 
 #ifdef HAVE_LIMITS
 #include <limits>
@@ -49,12 +50,13 @@ using std::vector;
 using std::list;
 using std::map;
 using std::string;
+using std::memset; // required for FD_ZERO
 
 const uint PScreen::MODIFIER_TO_MASK[] = {
     ShiftMask, LockMask, ControlMask,
     Mod1Mask, Mod2Mask, Mod3Mask, Mod4Mask, Mod5Mask
 };
-const uint PScreen::MODIFIER_TO_MASK_NUM = sizeof(PScreen::MODIFIER_TO_MASK[0]) / sizeof(PScreen::MODIFIER_TO_MASK);
+const uint PScreen::MODIFIER_TO_MASK_NUM = sizeof(PScreen::MODIFIER_TO_MASK) / sizeof(PScreen::MODIFIER_TO_MASK[0]);
 
 PScreen* PScreen::_instance = 0;
 
@@ -121,7 +123,6 @@ PScreen::PScreen(Display *dpy, bool honour_randr)
       _screen(-1), _depth(-1),
       _root(None), _visual(0), _colormap(None),
       _modifier_map(0),
-      _num_lock(0), _scroll_lock(0),
       _has_extension_shape(false), _event_shape(-1),
       _has_extension_xinerama(false),
       _has_extension_xrandr(false), _event_xrandr(-1),
@@ -700,3 +701,6 @@ PScreen::getKeycodeFromMask(uint mask)
 
     return 0;
 }
+
+uint PScreen::_num_lock = 0;
+uint PScreen::_scroll_lock = 0;
